@@ -110,11 +110,11 @@ void DLogManagerPrivate::updateLoggingRules()
 {
     QVariant var;
     // 4. 优先看 dsgConfig 是否默认值，其次 fallback 是否默认值
-    if (m_dsgConfig && m_dsgConfig->isInitializeSucceed() && !m_dsgConfig->rulesIsDefaultValue()) {
+    if (m_dsgConfig && m_dsgConfig->isInitializeSucceeded() && !m_dsgConfig->rulesIsDefaultValue()) {
         var = m_dsgConfig->rules();
-    } else if (m_fallbackConfig && m_dsgConfig->isInitializeSucceed() && !m_fallbackConfig->rulesIsDefaultValue()) {
+    } else if (m_fallbackConfig && m_fallbackConfig->isInitializeSucceeded() && !m_fallbackConfig->rulesIsDefaultValue()) {
         var = m_fallbackConfig->rules();
-    } else if (m_dsgConfig && m_dsgConfig->isInitializeSucceed()) {
+    } else if (m_dsgConfig && m_dsgConfig->isInitializeSucceeded()) {
         var = m_dsgConfig->rules();
     }
 
@@ -213,7 +213,7 @@ QString DLogManager::getlogFilePath()
         if (!QDir(cachePath).exists()) {
             QDir(cachePath).mkpath(cachePath);
         }
-        instance()->d_func()->m_logPath = instance()->joinPath(cachePath, QString("%1.log").arg(qApp->applicationName()));
+        instance()->d_func()->m_logPath = instance()->joinPath(cachePath, QString("%1.log").arg(QCoreApplication::applicationName()));
     }
 
     return QDir::toNativeSeparators(DLogManager::instance()->d_func()->m_logPath);
